--FILE PACKER - packs to gfp (uncompressed) gfz (compressed)
require "lfs"
require "json"

	local function loadFile(path)
		local file = io.open( path, "rb" )
		if file then
		   local contents = file:read( "*all" )
		   io.close( file )	
		   return contents
		end
	end

-- source_dir : full path to source directory
-- destination_file : full path to destination file, comprehensive of file name without extension ("mnt/sdcard/myfile")
--
function pack(source_dir, destination_file, zip)
	

	local data = {}
	data.filename = {}
	data.file = {}
	
	--read filepaths and their contents from source_directory
	for file in lfs.dir(source_dir) do 
		if lfs.attributes(file,"mode")== "directory" then --ignore subdirectories
			print("found dir, "..file)
		else 
			print("processing", source_dir..file)
			data.file[#data.file+1] = loadFile(source_dir..file)
			data.filename[#data.filename+1] = file
		end
	end

	local ext = ".gfp"
	local jsonString = json.encode( data )
	
	--compress jsonString with zlib if requested
	if zip then
		print("ZIP REQUESTED", zip)
		jsonString = zlib.compress(jsonString)
		--change file extension if using compression
		ext = ".gfz"
	end
	
	local path = destination_file..ext
	local outputfile = io.open( path, "wb" )
	if outputfile then
	   outputfile:write( jsonString )
	   io.close( outputfile )
	end

end

function depack(source_file, destination_dir)

	local function isDir(name)
		require "lfs"
		if type(name)~="string" then return false end
		local cd = lfs.currentdir()
		local is = lfs.chdir(name) and true or false
		lfs.chdir(cd)
		return is
	end
	
	local function getBasenameDirectoryExt(filename)
	   --"mnt/sdcard/myfile.lua"  -->mnt/sdcard/	myfile.lua	lua
	   return filename:match("(.-)([^\\/]-%.?([^%.\\/]*))$") 
	end
	
	if not isDir(destination_dir) then
		lfs.mkdir(destination_dir)
	end
	
	local sourcepath, sourcefile, sourceext = getBasenameDirectoryExt(source_file)
	print("FP UNPACK", sourcepath,  sourcefile, sourceext)
	local data 
	if sourceext == "gfz" then
		data = json.decode( zlib.decompress(loadFile(source_file)) )
	else
		data = json.decode( loadFile(source_file) )
	end	
		for i,file in ipairs( data.filename ) do
			print("writing", i,file)
			
			local path = destination_dir..file
			local outputfile = io.open( path, "wb" )
			if outputfile then
			   outputfile:write( data.file[i] )
			   io.close( outputfile )
			end
		end
end